local bar = _G[...]
local modName = ...

--Status Icon Bar

local component = bar:NewComponent("icons", "Frame")

function component:New(parent)
	local name = ("%s_%s_IconBar"):format(modName, parent.id)
	if _G[name] then
		return _G[name]
	end
	local frame = self:Bind(CreateFrame("Frame", name, parent.box))
	frame:SetFrameStrata("HIGH")
	frame:SetFrameLevel(200)
	frame.owner = parent
	frame:SetAttribute("unit", parent.id)
	frame:EnableMouse(false)
	frame.buttons = frame.buttons or {}
	frame.total = frame.total or 0
	frame.noMouse = true
	return frame
end

component.defaults = {
	columns = 8,
	alpha = 100,
	spacing = 0,
	frameLayer = 2,
	frameStrata = 2,
	width = 1,
	height = 1,
	scale = 130,
	anchor = "Right",
	x = -45,
	y = 2,
	IsTopToBottom = true,
	IsLeftToRight = false,
	enable = true,
	yFlex = 0,
	yArc = 0,
	yShift = 0,
	xFlex = 0,
	xArc = 0,
	xShift = 0,
	rings = 1,
	offset =  0,
	space =  0,
	spin =  0,
}

local layers = {
	"BACKGROUND",
	"LOW",
	"MEDIUM",
	"HIGH",
	"DIALOG",
	"FULLSCREEN",
	"FULLSCREEN_DIALOG",
	"TOOLTIP",
}

local function Bar_Layout(self)
	local arc = self.sets.ARC
	local yFlex, yArc, xFlex, xArc = self:GetArc()
	local numButtons = #self.buttons
	local sets = self.sets
	local cols = min(self.sets.columns, numButtons)
	local rows = ceil(numButtons / cols)
	local isLeftToRight = self.sets.IsLeftToRight
	local isTopToBottom = self.sets.IsTopToBottom
	-- grab base button sizes
	local size = 20 * (sets.scale/100)
	local w, h = size, size
	local spacing = self.sets.spacing
	local bW = w + spacing
	local bH = h + spacing
	local hSpacing = spacing
	local vSpacing = spacing
	--base bar size, no arc applied
	local width= (cols * w) + ((cols) + hSpacing)
	local height = (rows * h) + ((rows) + vSpacing)
	--changes the flex point of the vertical and horizontal arcs
	local xFlex = (((xFlex or 0)/100)/2 * (height))
	local yFlex = (((yFlex or 0)/100)/2 * (width))
	local a = (height/2)/((width/2)^2)
	local b = (width/2)/((height/2)^2)
	local xArc = xArc
	local yArc = yArc
	local xOff = 0
	local yOff = 0
	--align buttons based on center of frame
	local zeroX, zeroY = xOff + (bW)*(((cols)/2) - .5), yOff + (bH)*(((rows)/2) - .5)
	--flex buttons over center of the bar, with x and y curve offsets
	local R = 0
	local fX, fY = xOff + bW*0, yOff + bH*0
	local dX, dY = xOff + bW*(cols), yOff + bH*(rows)
	if cols < 3 then
		yArc = 0
	end
	if rows < 3 then
		xArc = 0
	end
	do
		fX, fY = (zeroX) - (fX - ((b*(((zeroY - math.abs(xFlex)) - fY)^2)) * ((xArc/100) * width))), zeroY - (fY - ((a*(((zeroX - math.abs(yFlex)) - fX)^2)) * ((yArc/100)*height)))
		fX, fY = (fX- zeroX)/2, (fY - zeroY)/2
		dX, dY = (zeroX) - (dX - ((b*(((zeroY - math.abs(xFlex)) - dY)^2)) * ((xArc/100) * width))), zeroY - (dY - ((a*(((zeroX - math.abs(yFlex)) - dX)^2)) * ((yArc/100)*height)))
		dX, dY = (dX- zeroX)/2, (dY - zeroY)/2
	end
	local flexX, flexY = dX, dY
	if (math.abs(fY) > math.abs(dY)) and (yFlex<0)then
		flexY = fY
	end
	if (math.abs(fX) > math.abs(dX)) and (xFlex>0) then
		flexX = fX
	end
	local maxX, maxY = 0, 0
	local minX, minY = 0, 0
	for i, button in ipairs(self.buttons) do
		button:SetSize(size, size)
		local row = floor((i - 1) / cols)
		if isTopToBottom then
			row = rows - (row + 1)
		else
			row = floor((i - 1) / cols)
		end
		local col = (i - 1) % cols
		if isLeftToRight then
			col = cols - (col + 1)
		else
			col = (i - 1) % cols
		end
		local X, Y = xOff + bW*col, yOff + bH*row
		X, Y = -(X - ((b*(((zeroY - xFlex) - Y)^2)) * ((xArc/100) * width))), -(Y - ((a*(((zeroX - yFlex) - X)^2)) * ((yArc/100)*height)))
		X = X - flexX
		Y = - (Y - flexY)
		if X > maxX then
			maxX = X
		end
		if X < minX then
			minX = X
		end
		if Y > maxY then
			maxY = Y
		end
		if Y < minY then
			minY = Y
		end
		button:SetParent(self:GetParent())
		button:ClearAllPoints()
		button:SetPoint("Center", self, X, Y)
	end
	local barWidth = (math.abs(maxX)+ math.abs(minX))+ bH - spacing
	local barHeight = (math.abs(maxY)+ math.abs(minY))+ bH - spacing
	self:SetSize(barWidth, barHeight)
end

local function Round_Layout(self)
	local rings, offset, spin, space, rowspace, rev, revOrder = self:GetSphere()
	spin = 90 - spin
	space = space - 2.92
	rowspace = rowspace * 2
	local numButtons = #self.buttons
	local cols = min(numButtons/((numButtons+1) - rings), numButtons)
	local rows = ceil(numButtons / cols)
	local isLeftToRight = self.sets.IsLeftToRight
	local isTopToBottom = self.sets.IsTopToBottom
	local size = 5 * (self.sets.scale/100)
	-- grab base button sizes
	local l, r, t, b = 0,0,0,0
	local bW, bH = size, size
	local pW, pH = 0,0
	local diameter =  math.sqrt((bW^2) + (bH^2)) + space
	local left, right, top, bottom
	if not self.center  then
		self.center = CreateFrame("Frame", nil, self)
		self.center:SetSize(20, 20)
		self.center:SetPoint("Center", self)
	end
	local base = 360/ceil(numButtons/rows)
	local circumference = ((diameter) * ceil(numButtons/rows)) / (math.pi)
	local left, right, top, bottom
	for i, button in ipairs(self.buttons) do
		if not revOrder then
			i = (numButtons +1) - i
		end
		local row = floor((i - 1) / cols)
		if row == (rows) then
			base = 360/floor(numButtons/rows)
		end
		row = (rows) - row
		local angle = -(.5*base-(offset*row)+offset) + (base*i)
		if (floor(row/2) == (row/2)) and rev then
			angle = -angle
		end
		angle = math.rad(angle - spin)
		local u = ((circumference + ((diameter*1.5) * (row-1)) + ((rowspace *row) - rowspace))/2) + self.sets.height
		local v = ((circumference + ((diameter*1.5) * (row-1)) + ((rowspace *row) - rowspace))/2) + self.sets.width

		
		local y, x = math.cos(angle)*u, -math.sin(angle)*v
		if isTopToBottom then
			y= -y
		end
		if isLeftToRight then
			x = -x
		end
		if (left and x < left) or (not left) then
			left = x
		end
		if (right and x > right) or (not right) then
			right = x
		end
		if (bottom and y < bottom) or (not bottom) then
			bottom = y
		end
		if (top and y > top) or (not top) then
			top = y
		end
		button:ClearAllPoints()
		button:SetPoint("Center", self.center, "Center", x , y)
	end
	diameter = (diameter/2) - 7.45
	local w, h = (diameter*2) - left + right , (diameter*2) - bottom + top
	self:SetSize( w + (pW*2) - space, h + (pH*2) - space)
	local x, y = (w/2) - (right + diameter) , (h/2) - (top + diameter)
	self.center:SetPoint("Center", self, x, y)
end

function component:Layout()
	local sets = self.sets
	if sets.enable ~=true then
		self:Hide()
		self.noUpdate = true
		return
	else
		self:Show()
		self.noUpdate = nil
	end
	self:ClearAllPoints()
	local x = sets.x/(sets.scale/100)
	local y = sets.y/(sets.scale/100)
	self:SetPoint(sets.anchor, self:GetParent(), x, y)
	if sets.scale < 25 then
		sets.scale = sets.scale *100
	end
	self:SetScale(sets.scale/100)
	self:SetAlpha(sets.alpha/100)
	local num = #self.ICONS
	for i = 1, num do
		self:getIcon(i)
	end
 	local numButtons = #self.buttons
	if numButtons < 1 then
		ButtonBar.proto.Layout(self)
		return
	end
	if self.sets.style == "BAR" then
		Bar_Layout(self)
	else
		Round_Layout(self)
	end
	local lay = layers[sets.frameStrata]
	self:SetFrameStrata(lay)
	self:SetFrameLevel(sets.frameLayer)
	
	self:Update()
end

function component:GetArc()
	local sets = self.sets
	local yFlex = sets.yFlex
	local yArc = sets.yArc
	local xFlex = sets.xFlex
	local xArc = sets.xArc
	return yFlex, yArc/2, xFlex, xArc/2
end

function component:GetSphere()
	local sets = self.sets
	if not sets then
		return
	end
	return (#component.ICONS+1) - sets.rings, sets.offset, sets.spin, sets.space, self.sets.spacing, sets.rev, sets.revOrder
end

function component:Update()
	if self.noUpdate or not self.buttons then
		return
	end
	local sets = self.sets
	local max = #self.ICONS
	self.total = 0
	local unit = UnitGUID(self.owner.id) --update, if the target changes
	for i = 1, max do
		local name, texture, coords = self.ICONS[i](self)
		if name and texture and (unit == self.lastUnit) then
			self:updateIcon(i, name, texture, coords)
		else
			self:resetIcon(i)
		end
	end
	self.lastUnit = unit
	self:updateTooltip()
end

component.Shapes = {
	["ROUND"] = {true, true, true, true},
	["SQUARE"] = {false, false, false, false},
	["CORNER-TOPLEFT"] = {false, false, false, true},
	["CORNER-TOPRIGHT"] = {false, false, true, false},
	["CORNER-BOTTOMLEFT"] = {false, true, false, false},
	["CORNER-BOTTOMRIGHT"] = {true, false, false, false},
	["SIDE-LEFT"] = {false, true, false, true},
	["SIDE-RIGHT"] = {true, false, true, false},
	["SIDE-TOP"] = {false, false, true, true},
	["SIDE-BOTTOM"] = {true, true, false, false},
	["TRICORNER-TOPLEFT"] = {false, true, true, true},
	["TRICORNER-TOPRIGHT"] = {true, false, true, true},
	["TRICORNER-BOTTOMLEFT"] = {true, true, false, true},
	["TRICORNER-BOTTOMRIGHT"] = {true, true, true, false},
}

function component:HasMouse()
	local index
	if self.lastMouse and MouseIsOver(self:getIcon(self.lastMouse)) then
		return true, self.lastMouse
	else
		for i = 1, self.total do
			local icon = self:getIcon(i)
			if MouseIsOver(icon) and self.isShown then
				self.hasMouse = i
				return true, i
			end
		end
	end
end

function component:updateTooltip()
	local hasMouse, index =  self:HasMouse()
	if hasMouse then
		if (index ~= self.lastMouse) and self:getIcon(index).isShown then
			GameTooltip:SetOwner(self)
			GameTooltip:SetText(self.owner.id, self:GetIndex(index))
			GameTooltip:ClearAllPoints()
			GameTooltip:SetPoint("BottomRight", self:getIcon(index), "TopLeft")
			self.lastMouse = index
		end
	else
		if GameTooltip:GetOwner() == self then
			self.hasMouse = nil
			self.lastMouse = nil
			GameTooltip:Hide()
			return
		end
	end
end

function component:GetIndex(i)
	local rev = i
	if self.sets.reverse then
		rev = (self.total + 1 ) - i
	end
	return rev, i
end

function component:updateIcon(i, name, texture, coords)
	local icon = self:getIcon(i)
	if (not icon) or (not name) then
		icon:SetTexture("")
		return
	end
	if type(texture) == "function" then
		icon.lastIcon = name
		texture(icon, self.owner.id)
	else
		icon.lastIcon = texture
		icon:SetTexture(texture)
		icon:SetTexCoord(unpack(coords))
	end
	if not icon.isShown then
		icon.isShown = true
	end
end

function component:ToggleTestMode()
	self.testMode = not self.testMode
	if self.testMode then
		self.ICONS = component.C_ons
	else
		self.ICONS = component.Cons
	end
	self:Update()
end

function component:resetIcon(i)
	local icon = self:getIcon(i)
	if icon.isShown then
		icon.lastIcon = nil
		icon:SetTexture("")
		icon:SetTexCoord(0,1,0,1)
		icon.isShown = nil
		if MouseIsOver(icon) then
			self.hasMouse = nil
			self.lastMouse = nil
		end
	end
end

function component:getIcon( i)
	if not self.buttons then
		return
	end
	local t = self.buttons[i] or self:CreateTexture(self:GetName().."Icon"..i, "ARTWORK")
	if not self.buttons[i] then
		self.buttons[i] = t
		t:SetParent(self)
		t:SetSize(20, 20)
		t.isShown = nil
		t.index = i
	end
	return t
end

component.C_ons = {
		function(self)
			return "Classification",  function(self, unit)
						self:SetAtlas("nameplates-icon-elite-gold");
			end
		end,
		function(self)
				index = 1;
				local left, right, top, bottom;
				local coordIncrement = RAID_TARGET_ICON_DIMENSION / RAID_TARGET_TEXTURE_DIMENSION;
				left = mod(index , RAID_TARGET_TEXTURE_COLUMNS) * coordIncrement;
				right = left + coordIncrement;
				top = floor(index / RAID_TARGET_TEXTURE_ROWS) * coordIncrement;
				bottom = top + coordIncrement;
				return "Raid Target", "Interface\\TargetingFrame\\UI-RaidTargetingIcons", {left, right, top, bottom}
		end,
		function(self)
				return "Resting", "Interface\\CharacterFrame\\UI-StateIcon", {.05, .45, .07, 0.46}
		end,
		function(self)
			local factionGroup = UnitFactionGroup(self.owner.id);
				return "PvP: FFA", "Interface\\TargetingFrame\\UI-PVP-FFA", {.1, .5, 0, .6}
			 end,
		function(self)
			return "Leader", "Interface\\GroupFrame\\UI-Group-LeaderIcon", {0, 1, 0, 1}
 		end,
 		function(self)
			return "Loot Master", "Interface\\GroupFrame\\UI-Group-MasterLooter", {0, 1, 0, 1}
 		end,
		function(self)
			local role = "TANK"
			return "Role: "..role, "Interface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES", {GetTexCoordsForRoleSmallCircle(role)}
	 	end,
		function(self)
				return "In Combat", "Interface\\CharacterFrame\\UI-StateIcon", {.5, 1, 0, 0.5}
		end,
		function(self)
			return "High Level", "Interface\\TargetingFrame\\UI-TargetingFrame-Skull", {0, 1, 0, 1}
 		end,
		function(self)
			return "Quest", "Interface\\TargetingFrame\\PortraitQuestBadge", {0, 1, 0, 1}
 		end,
		function(self)
				local petType = 3
				return "Battle Pet", "Interface\\TargetingFrame\\PetBadge-"..PET_TYPE_SUFFIX[petType], {0, 1, 0, 1}
 		end,
			}

component.Cons = {
		function(self)
			return "Classification",  function(self, unit)
				self:SetTexCoord(0,1,0,1)
				local classification = UnitClassification(unit);
				if ( classification == "elite" or classification == "worldboss" ) then
					self:SetAtlas("nameplates-icon-elite-gold");
				elseif ( classification == "rareelite" ) then
					self:SetAtlas("nameplates-icon-elite-silver");
				end
			end
		end,
		function(self)
			local index = GetRaidTargetIndex(self.owner.id);
			if ( index ) then
				index = index - 1;
				local left, right, top, bottom;
				local coordIncrement = RAID_TARGET_ICON_DIMENSION / RAID_TARGET_TEXTURE_DIMENSION;
				left = mod(index , RAID_TARGET_TEXTURE_COLUMNS) * coordIncrement;
				right = left + coordIncrement;
				top = floor(index / RAID_TARGET_TEXTURE_ROWS) * coordIncrement;
				bottom = top + coordIncrement;
				return "Raid Target", "Interface\\TargetingFrame\\UI-RaidTargetingIcons", {left, right, top, bottom}
			end
		end,
		function(self)
			if ( (self.owner.id == "player") and IsResting() ) then
				return "Resting", "Interface\\CharacterFrame\\UI-StateIcon", {.05, .45, .07, 0.46}
			end
			 		end,
		function(self)
			local factionGroup = UnitFactionGroup(self.owner.id);
			if ( UnitIsPVPFreeForAll(self.owner.id) ) then
				return "PvP: FFA", "Interface\\TargetingFrame\\UI-PVP-FFA", {.1, .5, 0, .6}
			elseif ( factionGroup and factionGroup ~= "Neutral" and UnitIsPVP(self.owner.id) ) then
				return "PvP", "Interface\\TargetingFrame\\UI-PVP-"..factionGroup, {.025, .6, .025, .6}
			end
			 		end,
		function(self)
			if ( UnitIsGroupLeader(self.owner.id) ) then
				if ( HasLFGRestrictions() ) then
					return "Guide", "Interface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES", {0, 0.296875, 0.015625, 0.3125}
				else
					return "Leader", "Interface\\GroupFrame\\UI-Group-LeaderIcon", {0, 1, 0, 1}
				end
			end
 		end,
 		function(self)
			local lootMethod
			local lootMaster
			lootMethod, lootMaster = GetLootMethod()
			if ( lootMaster == 0 and IsInGroup() ) then
				return "Loot Master", "Interface\\GroupFrame\\UI-Group-MasterLooter", {0, 1, 0, 1}
			end
 		end,
		function(self)
			local role = UnitGroupRolesAssigned(self.owner.id)
			if ( role == "TANK" or role == "HEALER" or role == "DAMAGER") then
				return "Role: "..role, "Interface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES", {GetTexCoordsForRoleSmallCircle(role)}
			end
	 	end,
		function(self)
			if (UnitAffectingCombat(self.owner.id)) then
				return "In Combat", "Interface\\CharacterFrame\\UI-StateIcon", {.5, 1, 0, 0.5}
			end
		end,
		function(self)
			local targetLevel = UnitLevel(self.owner.id)
			if (not  UnitIsCorpse(self.owner.id)) and (not ( targetLevel > 0 )) and (not ( UnitIsWildBattlePet(self.owner.id) and UnitIsBattlePetCompanion(self.owner.id)))  then
				return "High Level", "Interface\\TargetingFrame\\UI-TargetingFrame-Skull", {0, 1, 0, 1}
			end
 		end,
		function(self)
			if (UnitIsQuestBoss(self.owner.id)) then
				return "Quest", "Interface\\TargetingFrame\\PortraitQuestBadge", {0, 1, 0, 1}
			end
 		end,
		function(self)
			if ( UnitIsWildBattlePet(self.owner.id) or UnitIsBattlePetCompanion(self.owner.id) ) then
				local petType = UnitBattlePetType(self.owner.id)
				return "Battle Pet", "Interface\\TargetingFrame\\PetBadge-"..PET_TYPE_SUFFIX[petType], {0, 1, 0, 1}
			end
 		end,
}

component.ICONS = component.Cons

component.options = {
					{
						name = "Enable",
						kind = "CheckButton",
						key = "enable",
						panel = "basic",
					},
					{
						name = "Reverse",
						kind = "CheckButton",
						key = "reverse",
						panel = "basic",
					},
					{
						name = "isTopToBottom",
						kind = "CheckButton",
						key = "isTopToBottom",
						panel = "basic",
					},
					{
						name = "isLeftToRight",
						kind = "CheckButton",
						key = "isLeftToRight",
						panel = "basic",
					},
					{
						name = "Opacity",
						kind = "Slider",
						key = "alpha",
						min = 0,
						max = 100,
						panel = "basic",
					},

					{
						kind = "Button",
						name = "Enable Bar",
						handler = "icons",
						func = function(owner)
							owner.sets.style = "BAR"
							owner:Layout()
						end,
						panel = "bar",
					},
				   {
						name = "Y Arc",
						kind = "Slider",
						key = "yArc",
						min = -100,
						max = 100,
						panel = "bar",
					},
					{
						name = "Y Flex",
						kind = "Slider",
						key = "yFlex",
						min = -100,
						max = 100,
						panel = "bar",
					},
					{
						name = "X Arc",
						kind = "Slider",
						key = "xArc",
						min = -100,
						max = 100,
						panel = "bar",
					},
					{
						name = "X Flex",
						kind = "Slider",
						key = "xFlex",
						min = -100,
						max = 100,
						panel = "bar",
					},
					{
						name = "Columns",
						kind = "Slider",
						key = "columns",
						min = 1,
						max = #component.ICONS,
						panel = "bar",
					},
					{
						name = "Spacing",
						kind = "Slider",
						key = "spacing",
						min = -13,
						max = 36,
						panel = "bar",
					},

					{
						kind = "Button",
						name = "Enable Round",
						handler = "icons",
						func = function(owner)
							owner.sets.style = "ROUND"
							owner:Layout()
						end,
						panel = "round",
					},
					{
						name = "Reverse Order",
						kind = "CheckButton",
						key = "reverse",
						panel = "round",
					},
					{
						name = "Reverse Spin",
						kind = "CheckButton",
						key = "s",
						panel = "round",
					},
					{
						name = "Rings",
						kind = "Slider",
						key = "rings",
						min = 1,
						max = #component.ICONS,
						panel = "round",
					},
					{
						name = "Ring Space",
						kind = "Slider",
						key = "space",
						min = -13,
						max = 30,
						panel = "round",
					},
					{
						name = "Height",
						kind = "Slider",
						key = "height",
						min = -20,
						max = 400,
						panel = "round",
					},
					{
						name = "Width",
						kind = "Slider",
						key = "width",
						min = -20,
						max = 400,
						panel = "round",
					},
					{
						name = "Spacing",
						kind = "Slider",
						key = "spacing",
						min = -50,
						max = 100,
						panel = "round",
					},
					{
						name = "Spin",
						kind = "Slider",
						key = "spin",
						min = -180,
						max = 180,
						panel = "round",
					},
					{
						name = "Offset",
						kind = "Slider",
						key = "offset",
						min = -180,
						max = 180,
						panel = "round",
					},

					{
						kind = "Button",
						name = "Test Mode",
						handler = "icons",
						func = function(owner)
							owner:ToggleTestMode()
						end,
						panel = "advanced",
					},
					{
						name = "Frame Layer",
						kind = "Slider",
						key = "frameLayer",
						min = 1,
						max = 100,
						panel = "advanced",
					},
					{
						name = "Frame Strata",
						kind = "Slider",
						key = "frameStrata",
						min = 1,
						max = 8,
						panel = "advanced",
					},
					{
						name = "Anchor",
						kind = "Menu",
						key = "anchor",
						table = {
							"TopLeft",
							"Top",
							"TopRight",
							"Right",
							"BottomRight",
							"Bottom",
							"BottomLeft",
							"Left",
							"Center",
						},
						panel = "advanced",
					},
					{
						name = "X Offset",
						kind = "Slider",
						key = "x",
						min = -400,
						max = 400,
						panel = "advanced",
					},
					{
						name = "Y Offset",
						kind = "Slider",
						key = "y",
						min = -400,
						max = 400,
						panel = "advanced",
					},
					{
						name = "Scale",
						kind = "Slider",
						key = "scale",
						min = 25,
						max = 250,
						panel = "advanced",
					},

		}

function component:CreateMenu(menu)
	return bar.Menu.NewMenu(self, menu, "Icons", self.options)
end
