local player_class = select(2, UnitClass("player"))
local player_spec = GetSpecialization()
local max_points = 9
local Resource

local function applyTexture(self, Resource, icon, state) --1 = bg Only, 2 = Show All, 3 = Hide All
		local h, w = icon:GetSize()
		Resource.getBase(icon, w, h)
		icon.base:SetAlpha(1)
		icon:Show()
		Resource.getHighlight(icon, w, h)
		Resource.getShine(icon, w, h)
		if state == 3 then
			if (not icon.hideAll) or (icon:IsShown()) then
				icon.hideAll = true
				icon:Hide()
			end
		else
			if icon.hideAll then
				icon:Show()
			end
			if state == 2 then
				if not icon.highlightShown then
					local fadeInfo = {};
					fadeInfo.mode = "IN";
					fadeInfo.timeToFade = COMBOFRAME_HIGHLIGHT_FADE_IN;
					fadeInfo.finishedFunc = ComboPointShineFadeIn;
					fadeInfo.finishedArg1 = icon.shine;
					UIFrameFade(icon.highlight, fadeInfo)
					icon.highlightShown = true
				end
			else
				if icon.highlightShown then
					local fadeInfo = {};
					fadeInfo.mode = "OUT";
					fadeInfo.timeToFade = COMBOFRAME_HIGHLIGHT_FADE_IN;
					UIFrameFade(icon.highlight, fadeInfo)
					icon.highlightShown = nil
				end
			end
		end
	end


local sources = {}

do --rouge, druid, sometimes vehicle
	local Resource = {}
	max_points = 9
	Resource.getBase = function(icon, w, h)
		local texture, coords = [[Interface\ComboFrame\ComboPoint]], {0, 0.375, 0, 1}
		if not icon.base.set then
			icon.base:SetTexture(texture)
			icon.base:SetTexCoord(unpack(coords))
			icon.base:SetPoint("TopLeft", 0, 0)
			icon.base.set = true
		end
		if (icon.w ~= w) or (icon.h ~= h) then
			icon.base:SetSize((12/12) * h, (16/12) * w)
			icon.w = w
			icon.h = h
		end
	end
	Resource.getHighlight = function(icon, w, h)
		local texture, coords =  [[Interface\ComboFrame\ComboPoint]], {0.375, 0.5625, 0, 1}
		if not icon.highlight.set then
			icon.highlight:SetTexture(texture)
			icon.highlight:SetTexCoord(unpack(coords))
			icon.highlight:SetAlpha(0)
			icon.highlight.set = true
		end
		
		if (icon.lightW ~= w) or (icon.lightH ~= h) then
			icon.highlight:SetSize((8/12) * h, (16/12) * w)
			icon.highlight:SetPoint("TopLeft", (2/12) * w, 0)
			icon.lightW = w
			icon.lightH = h
		end
		
	end
	Resource.getShine = function(icon, w, h)
		local texture, coords = [[Interface\ComboFrame\ComboPoint]], {0.5625, 1, 0, 1}
		if not icon.shine.set then
			icon.shine:SetTexture(texture)
			icon.shine:SetTexCoord(unpack(coords))
			icon.shine:SetVertexColor(1,1,1,0)
			icon.shine:SetBlendMode("ADD")
			icon.shine:SetAlpha(0)
			icon.shine.set = true
		end
		if (icon.shineW ~= w) or (icon.h ~= shineH) then
			icon.shine:SetSize((14/12) * h, (16/12) * w)
			icon.shine:SetPoint("TopLeft", 0, (4/12) * h)
			icon.shineW = w
			icon.shineH = h
		end
	end
	Resource.applyTexture = applyTexture
	Resource.update = function(self)
		local icons = self.icons
		local is_rogue = player_class == "ROGUE"
		local is_druid = player_class == "DRUID"


		local hasVehicle = UnitHasVehicleUI(self.owner.id)

		local power = hasVehicle and GetComboPoints("vehicle", self.owner.id) or UnitPower(self.owner.id, SPELL_POWER_COMBO_POINTS)

		if (self:GetParent().owner.id == "pet" and not hasVehicle)
		or (self.owner.id ~= "player") 
		or ((power == 0) and ((not is_rogue and not is_druid) or (is_druid and GetShapeshiftFormID() ~= CAT_FORM))) then
			return
		end

		local maxPower = UnitPowerMax(self.owner.id, SPELL_POWER_COMBO_POINTS)
		if maxPower == 0 then maxPower = 5 end

		local icons = self.icons
		for i, b in pairs(icons) do
			b.index = i	
			if i <= maxPower then
				if i >= power then
					Resource.applyTexture(self, Resource, b, 1) --bg only
				else
					Resource.applyTexture(self, Resource, b, 2) --show all
				end
			else
				Resource.applyTexture(self, Resource, b, 3) -- Hide
			end
		end
	end
	sources.ComboPoints = Resource
end

do --mage
	local Resource = {}
	max_points = 4
	Resource.getBase = function(icon, w, h)
		local texture =  [[Mage-ArcaneCharge]]
		if not icon.base.set then
			icon.base:SetAtlas(texture)
			icon.base:SetAllPoints(icon)
			
			icon.base:SetAlpha(.6)
			icon.base.set = true
		end
	end
	Resource.getHighlight = function(icon, w, h)
		local texture =  [[Mage-ArcaneCharge]]
		if not icon.highlight.set then
			icon.highlight:SetAtlas(texture)
			icon.highlight:SetBlendMode("ADD")
			icon.highlight:SetAllPoints(icon)
			
			icon.highlight:SetAlpha(0)
			icon.highlight.set = true
		end
	end
	Resource.getShine = function(icon, w, h)
		local texture =  [[Mage-ArcaneCharge-CircleGlow]]
		if not icon.shine.set then
			icon.shine:SetAtlas(texture)
			icon.shine:SetBlendMode("ADD")
			icon.shine:SetAllPoints(icon)
			
			icon.shine:SetAlpha(0)
			icon.shine.set = true
		end
	end
	Resource.applyTexture = applyTexture
	Resource.update = function(self)
		local icons = self.icons
		local power = UnitPower(self.owner.id, SPELL_POWER_ARCANE_CHARGES);
	
		local maxPower = UnitPowerMax(self.owner.id, SPELL_POWER_ARCANE_CHARGES )
		
		local icons = self.icons
		for i, b in pairs(icons) do
			b.index = i
			if i <= maxPower then
				if i > power then
					Resource.applyTexture(self, Resource, b, 1) --bg only
				else
					Resource.applyTexture(self, Resource, b, 2) --show all
				end
			else
				Resource.applyTexture(self, Resource, b, 3) -- Hide
			end
		end
	end

	sources.ArcaneCharges = Resource
end

do --monk
	local Resource = {}
	max_points = 5
	Resource.getBase = function(icon, w, h)
		local texture, coords = [[Interface\ComboFrame\ComboPoint]], {0, 0.375, 0, 1}
		if not icon.base.set then
			icon.base:SetTexture(texture)
			icon.base:SetTexCoord(unpack(coords))
			icon.base:SetPoint("TopLeft", 0, 0)
			icon.base.set = true
		end
		if (icon.w ~= w) or (icon.h ~= h) then
			icon.base:SetSize((12/12) * h, (16/12) * w)
			icon.w = w
			icon.h = h
		end
	end
	Resource.getHighlight = function(icon, w, h)
		local texture =  [[MonkUI-LightOrb]]
		if not icon.highlight.set then
			icon.highlight:SetAtlas(texture)
			icon.highlight:SetAlpha(0)
			icon.highlight:SetAllPoints(icon)
			icon.highlight.set = true
		end
	end
	Resource.getShine = function(icon, w, h)
		local texture, coords = [[Interface\ComboFrame\ComboPoint]], {0.5625, 1, 0, 1}
		if not icon.shine.set then
			icon.shine:SetTexture(texture)
			icon.shine:SetTexCoord(unpack(coords))
			icon.shine:SetVertexColor(1,1,1,0)
			icon.shine:SetBlendMode("ADD")
			icon.shine:SetAlpha(0)
			icon.shine.set = true
		end
		if (icon.shineW ~= w) or (icon.h ~= shineH) then
			icon.shine:SetSize((14/12) * h, (16/12) * w)
			icon.shine:SetPoint("TopLeft", 0, (4/12) * h)
			icon.shineW = w
			icon.shineH = h
		end
	end
	Resource.applyTexture = applyTexture
	Resource.update = function(self)
		local icons = self.icons
		local power = UnitPower(self.owner.id, SPELL_POWER_CHI);
	
		local maxPower = UnitPowerMax(self.owner.id, SPELL_POWER_CHI )
		
		local icons = self.icons
		for i, b in pairs(icons) do
			b.index = i
			if i <= maxPower then
				if i > power then
					Resource.applyTexture(self, Resource, b, 1) --bg only
				else
					Resource.applyTexture(self, Resource, b, 2) --show all
				end
			else
				Resource.applyTexture(self, Resource, b, 3) -- Hide
			end
		end
	end
	sources.Chi = Resource
end

do --warlock
	local Resource = {}
	max_points = 5
	Resource.getBase = function(icon, w, h, scale)
		local texture, coords = [[Interface\PlayerFrame\UI-WarlockShard]], {.01562500, 0.82812500, 0.60937500, .83593750}
		if not icon.base.set then
			icon.base:SetTexture(texture)
			icon.base:SetTexCoord(unpack(coords))
			icon.base.set = true
		end
		if (icon.w ~= w) or (icon.h ~= h) or (icon.baseScale ~= scale) then
			icon.base:SetSize(52, 29)
			icon.base:SetPoint("TopLeft", -5, 3)
			icon.w = w
			icon.h = h
			icon.baseScale = scale
		end
	end
	Resource.getHighlight = function(icon, w, h, scale)
		local texture, coords = [[Interface\PlayerFrame\UI-WarlockShard]], {0.01562500, .28125000, 0.00781250, 0.13281250}
		if not icon.highlight.set then
			icon.highlight:SetTexture(texture)
			icon.highlight:SetTexCoord(unpack(coords))
			icon.highlight:SetAlpha(0)
			icon.highlight.set = true
		end
		if (icon.highlightW ~= w) or (icon.highlightH ~= h) or (icon.highlightScale ~= scale) then
			icon.highlight:SetSize(17, 16)
			icon.highlight:SetPoint("TopLeft", 3, -2)
			icon.highlightW = w
			icon.highlightH = h
			icon.highlightScale = scale
		end
		
	end
	Resource.getShine = function(icon, w, h, scale)	
		local texture, coords = [[Interface\PlayerFrame\UI-WarlockShard]], {0.01562500, .42187500, 0.14843750, 0.32812500}
		if not icon.shine.set then
			icon.shine:SetTexture(texture)
			icon.shine:SetTexCoord(unpack(coords))
			icon.shine:SetAlpha(0)
			icon.shine:SetBlendMode("ADD")
			icon.shine.set = true
		end
		if (icon.shineW ~= w) or (icon.shineH ~= h) or (icon.shineScale ~= scale) then
			icon.shine:SetSize((26), (23))
			icon.shine:SetPoint("TopLeft", -2, 1)
			icon.shineW = w
			icon.shineH = h
			icon.shineScale = scale
		end
	end
	Resource.applyTexture = applyTexture
	Resource.update = function(self)
		local icons = self.icons
		local power = UnitPower(self.owner.id, SPELL_POWER_SOUL_SHARDS);
	
		local maxPower = UnitPowerMax(self.owner.id, SPELL_POWER_SOUL_SHARDS )
		
		local icons = self.icons
		for i, b in pairs(icons) do
			b.index = i
			if i <= maxPower then
				if i > power then
					Resource.applyTexture(self, Resource, b, 1) --bg only
				else
					Resource.applyTexture(self, Resource, b, 2) --show all
				end
			else
				Resource.applyTexture(self, Resource, b, 3) -- Hide
			end
		end
	end
	sources.Shards = Resource
end

do --death Knight
	local Resource = {}
	max_points = 7	 
	Resource.getBase = function(icon, w, h)
		local texture, coords = [[Interface\PlayerFrame\UI-PlayerFrame-Deathknight-Ring]], {0,1,0,1}
		if not icon.base.set then
			icon.base:SetTexture(texture)
			icon.base:SetTexCoord(unpack(coords))
			icon.base:SetVertexColor(.6, .6, .6, 1)
			icon.base.set = true
		end
		if (icon.w ~= w) or (icon.h ~= h) then
			icon.base:SetSize(24, 24)
			
			icon.base:SetPoint("TopLeft", 0, 1)
			icon.w = w
			icon.h = h
		end
	end
	Resource.getHighlight = function(icon, w, h)
		local texture, coords =  [[Interface\PlayerFrame\UI-PlayerFrame-Deathknight-SingleRune]], {0,1,0,1}
		if not icon.highlight.set then
			icon.highlight:SetTexture(texture)
			icon.highlight:SetTexCoord(unpack(coords))
			icon.highlight:SetAlpha(0)
			icon.highlight.set = true
		end
		
		if (icon.lightW ~= w) or (icon.lightH ~= h) then
			icon.highlight:SetSize(24, 24)
			
			icon.highlight:SetPoint("TopLeft", 0, 1)
			icon.lightW = w
			icon.lightH = h
		end
		
	end
	Resource.getShine = function(icon, w, h)
		local texture, coords = [[Interface\ComboFrame\ComboPoint]], {0.5625, 1, 0, 1}
		if not icon.shine.set then
			icon.shine:SetTexture(texture)
			icon.shine:SetTexCoord(unpack(coords))
			icon.shine:SetVertexColor(1,1,1,0)
			icon.shine:SetBlendMode("ADD")
			icon.shine:SetAlpha(0)
			icon.shine.set = true
		end
		if (icon.shineW ~= w) or (icon.h ~= shineH) then
			icon.shine:SetSize((14/12) * h, (16/12) * w)
			icon.shine:SetPoint("TopLeft", 0, (4/12) * h)
			icon.shineW = w
			icon.shineH = h
		end
	end
	Resource.applyTexture = applyTexture
	Resource.update = function(self)
		local icons = self.icons
		local maxPower = UnitPowerMax(self.owner.id, SPELL_POWER_RUNES)
		local icons = self.icons
		for i, icon in pairs(icons) do
			local start, duration, runeReady = GetRuneCooldown(i)
			icon.index = i
			if i <= maxPower then
				if runeReady then
					Resource.applyTexture(self, Resource, icon, 2) --show all
					icon.cooldown:Hide()
				else
					Resource.applyTexture(self, Resource, icon, 2) --show all
					if start then
						CooldownFrame_Set(icon.cooldown, start, duration, true, true)
					end
				end
			else
				Resource.applyTexture(self, Resource, icon, 3) -- Hide
			end
		end

	end
	sources.Runes = Resource
end

do --Paladin
	local Resource = {}
	max_points = 5	
	Resource.getBase = function(icon, w, h)
		local texture, coords
		local index = icon.index
		local w, h
		if index ==1 then
			texture, coords =  [[Interface\PlayerFrame\PaladinPowerTextures]], {0.00390625, 0.14453125, 0.78906250, 0.96093750}
			w, h = 36, 22
		elseif index == 2 then
			texture, coords =  [[Interface\PlayerFrame\PaladinPowerTextures]], {0.15234375, 0.27343750, 0.78906250, 0.92187500}
			w, h = 31, 17
		elseif index == 3 then
			texture, coords =  [[Interface\PlayerFrame\PaladinPowerTextures]], {0.28125000, 0.38671875, 0.64843750, 0.81250000}
			w, h = 27, 21

		elseif index == 4 then
			texture, coords =  [[Interface\PlayerFrame\PaladinPowerTextures]], {0.28125000, 0.38671875, 0.82812500, 0.92187500}
			w, h = 27, 12

		elseif index == 5 then
			texture, coords =  [[Interface\PlayerFrame\PaladinPowerTextures]], {0.39453125, 0.49609375, 0.64843750, 0.74218750}
			w, h = 26, 12

		end
		
		
		if not icon.base.set then
			icon.base:SetTexture(texture)
			icon.base:SetTexCoord(unpack(coords))
			icon.base:SetVertexColor(0,0,0)
			icon.base.set = true
		end
		
		if (icon.baseW ~= w) or (icon.baseH ~= h) then
			icon.base:SetSize(h, w)
			
			icon.base:SetAllPoints(icon)
			icon.baseW = w
			icon.baseH = h
		end
		
	end
	Resource.getHighlight = function(icon, w, h)
		local texture, coords
		local index = icon.index
		local w, h
		if index ==1 then
			texture, coords =  [[Interface\PlayerFrame\PaladinPowerTextures]], {0.00390625, 0.14453125, 0.78906250, 0.96093750}
			w, h = 36, 22
		elseif index == 2 then
			texture, coords =  [[Interface\PlayerFrame\PaladinPowerTextures]], {0.15234375, 0.27343750, 0.78906250, 0.92187500}
			w, h = 31, 17
		elseif index == 3 then
			texture, coords =  [[Interface\PlayerFrame\PaladinPowerTextures]], {0.28125000, 0.38671875, 0.64843750, 0.81250000}
			w, h = 27, 21

		elseif index == 4 then
			texture, coords =  [[Interface\PlayerFrame\PaladinPowerTextures]], {0.28125000, 0.38671875, 0.82812500, 0.92187500}
			w, h = 27, 12

		elseif index == 5 then
			texture, coords =  [[Interface\PlayerFrame\PaladinPowerTextures]], {0.39453125, 0.49609375, 0.64843750, 0.74218750}
			w, h = 26, 12

		end
		
		
		if not icon.highlight.set then
			icon.highlight:SetTexture(texture)
			icon.highlight:SetTexCoord(unpack(coords))
			icon.highlight:SetAlpha(0)
			icon.highlight.set = true
		end
		
		if (icon.lightW ~= w) or (icon.lightH ~= h) then
			icon.highlight:SetSize(h, w)
			
			icon.highlight:SetAllPoints(icon)
			icon.lightW = w
			icon.lightH = h
		end
		
	end
	Resource.getShine = function(icon, w, h)
		local texture, coords
		local index = icon.index
		local w, h
		if index ==1 then
			texture, coords =  [[Interface\PlayerFrame\PaladinPowerTextures]], {0.00390625, 0.14453125, 0.78906250, 0.96093750}
			w, h = 36, 22
		elseif index == 2 then
			texture, coords =  [[Interface\PlayerFrame\PaladinPowerTextures]], {0.15234375, 0.27343750, 0.78906250, 0.92187500}
			w, h = 31, 17
		elseif index == 3 then
			texture, coords =  [[Interface\PlayerFrame\PaladinPowerTextures]], {0.28125000, 0.38671875, 0.64843750, 0.81250000}
			w, h = 27, 21

		elseif index == 4 then
			texture, coords =  [[Interface\PlayerFrame\PaladinPowerTextures]], {0.28125000, 0.38671875, 0.82812500, 0.92187500}
			w, h = 27, 12

		elseif index == 5 then
			texture, coords =  [[Interface\PlayerFrame\PaladinPowerTextures]], {0.39453125, 0.49609375, 0.64843750, 0.74218750}
			w, h = 26, 12

		end
		
		
		if not icon.shine.set then
			icon.shine:SetTexture(texture)
			icon.shine:SetTexCoord(unpack(coords))
			icon.shine:SetAlpha(0)
			icon.shine:SetBlendMode("ADD")

			icon.shine.set = true
		end
		
		if (icon.shineW ~= w) or (icon.shineH ~= h) then
			icon.shine:SetSize(h, w)
			
			icon.shine:SetAllPoints(icon)
			icon.shineW = w
			icon.shineH = h
		end
	end
	Resource.applyTexture = applyTexture
	Resource.update = function(self)
		local icons = self.icons
		local maxPower = UnitPowerMax(self.owner.id, SPELL_POWER_HOLY_POWER)
		local power = UnitPower(self.owner.id, SPELL_POWER_HOLY_POWER)
		
		local icons = self.icons
		for i, icon in pairs(icons) do
		icon.base:SetAlpha(1)
		
		icon.base:Show()
			icon.index = i
			if i <= maxPower then
				if i > power then
					Resource.applyTexture(self, Resource, icon, 1) --bg only
					
				else
					Resource.applyTexture(self, Resource, icon, 2) --show all
				end
			else
				Resource.applyTexture(self, Resource, icon, 3) -- Hide
			end
		end

	end
	sources.HolyPower = Resource
end 

local CleanResource
local function UpdateResource(self, unit)
	if ((player_class == "DRUID") and (GetShapeshiftFormID() == CAT_FORM)) then
		return sources.ComboPoints
	elseif (player_class == "ROUGE") or UnitHasVehicleUI(unit) then
		return sources.ComboPoints
	elseif (player_class == "MAGE") then
		return sources.ArcaneCharges
		
	elseif (player_class == "MONK") then
		return sources.Chi

	elseif (player_class == "WARLOCK") then
		return sources.Shards

	elseif (player_class == "DEATHKNIGHT") then
		return sources.Runes

	elseif (player_class == "PALADIN") then
		return sources.HolyPower
	end
end 

function CleanResource(icons)

	for i, icon in pairs(icons) do
		icon.base:SetTexture("")
		icon.base:SetAlpha(0)
	    icon.base:Hide()
		icon.base.set = nil
		
		icon.highlight:SetTexture("")
		icon.highlight:SetAlpha(0)
		icon.highlight:Hide()
		icon.highlight.set = nil
		
		icon.shine:SetTexture("")
		icon.shine:SetAlpha(0)
		icon.shine:Hide()
		icon.shine.set = nil
		
		icon.cooldown:SetAlpha(0)
		icon.cooldown:Hide()
		icon.cooldown.set = nil
	end
end

local  function DisplayResource(icons)
	for i, icon in pairs(icons) do
		icon.base:SetAlpha(1)
	    icon.base:Show()

		--icon.highlight:SetAlpha(1)
		icon.highlight:Show()
		
		--icon.shine:SetAlpha(1)
		icon.shine:Show()
		
		--icon.cooldown:SetAlpha(1)
		icon.cooldown:Show()
	end
end

local bar = _G[...]
local modName = ...

--Buff Bar
local component = bar:NewComponent('resource', 'Frame')

function component:New(parent)
	if parent.id ~= "player" then
		return
	end
	local name = ('%s_%s_resourceBar'):format(modName, parent.id)
	if _G[name] then
		return _G[name]
	end
	local bar = self:Bind(CreateFrame('Frame', name, parent.box))
	bar.owner = parent

	bar.drop = bar.drop or CreateFrame('Frame', nil, bar)
	bar.drop:SetAllPoints(bar)

	bar.kind = 'buff'

	bar.icons = {}
	local icon
	for i = 1, max_points do
		icon = CreateFrame("Frame", nil, bar)
		icon.base = icon:CreateTexture(nil, 'BACKGROUND')
			   
		icon.highlight = icon:CreateTexture(nil, 'ARTWORK')
		icon.shine = icon:CreateTexture(nil, 'OVERLAY')
		
		icon.cooldown = CreateFrame("Cooldown", nil, icon, "CooldownFrameTemplate")
		icon.cooldown:SetHideCountdownNumbers(true)
		icon.cooldown:Hide()
		icon.cooldown:SetPoint("Topleft", 0,1)
		icon.cooldown:SetSize(15,15)
		
		icon:SetSize(20, 20)
		tinsert(bar.icons, icon)
	end

	bar:SetSize(30, 30)

	return bar
end

component.defaults = {
	alpha = 100,
	spacing = 0,
	showBG = false,
	isTopToBottom = false,
	enable = true,
	anchor = 'BottomLeft',
	x = 12,
	y = 6,
	scale = 64,
	columns = 9,
	angle = 0,
	compress = 0,
	width = 50,
	height = 50,
	style = "Bar",
	rotateIcons = 0,
	color = {
		r = 1,
		g = 1,
		b = 1,
		a = 1,	
	}
}

function component:Load()
	if self.owner.id ~= "player" then
		return
	end

	self:EnableMouse(false)
	self.noMouse = true
	self.id = self.owner.id
end

function component:updatePosition(i, button)
	local num = #self.icons
	local base = (360/num) - (((360/num)) * (-(self.sets.spacing)/100))
	local angle = math.rad(self.sets.angle + ((base) *i)- (base))
	local x, y, q = math.cos(angle), math.sin(angle), 1
	if x < 0 then q = q + 1 end
	if y > 0 then q = q + 2 end
	--local quadTable = self.Shapes[self.sets.style]
	local w = self.sets.width/2
	local h = self.sets.height/2
	if self.sets.style == "ROUND" then
		x, y = x*w, y*h
	elseif self.sets.style == "SQUARE" then
		x, y = x*w, y*h
		local ang = self.sets.angle + ((base) *i)- (base)
		if (ang <= 45) or (ang >= 315)then
			x = w
		elseif ang <= 135 then
			y = h
		elseif ang <=225  then
			x = -w
		elseif ang <= 315 then
			y = -h
		end
	else
		x = x*w
		y = y*h
	end
	button:ClearAllPoints()
	if self.sets.isLeftToRight then
		x = -x
	end
	if self.sets.isTopToBottom then
		y = -y
	end
	button:SetPoint('CENTER', self, 'CENTER', x, y)
end

function component:Layout()
	local sets = self.sets
	if sets.enable ~=true then
--		self:Hide()
		self.noUpdate = true
		return
	else
		self:Show()
		self.noUpdate = nil
	end
	
	print(33)
	
	self:ClearAllPoints()
	local x = sets.x/(sets.scale/100)
	local y = sets.y/(sets.scale/100)
	self:SetPoint(sets.anchor, self:GetParent(), x, y)

	local w, h = self.icons[1]:GetSize()
	w, h =  w + sets.spacing,  h + sets.spacing
	
	local num = #self.icons
	
	local rows = ceil(num / sets.columns)
	local newWidth =  w * sets.columns - sets.spacing
	local newHeight = h * rows	- sets.spacing
	if sets.style == 'Bar' then
		self:SetSize(newWidth, newHeight)
		local cols, rows = sets.columns, rows
		local isLeftToRight = sets.isLeftToRight
		local isTopToBottom = sets.isTopToBottom
		for i, icon in pairs(self.icons) do
			local col, row = (cols-1) - (i-1) % cols, rows - ceil(i / cols)
			if isLeftToRight then
				col = (i-1) % cols
			end
			if isTopToBottom then
				row = ceil(i / cols) - 1
			end
			icon:ClearAllPoints()
			icon:SetPoint('TOPLEFT', w*col, -(h*row))
			icon:Show()
			icon:SetScale(sets.scale/100)
		end
	else--round???
		self:SetSize(sets.width, sets.height)
	
		for i, icon in pairs(self.icons) do
			self:updatePosition(i, icon)
		end
	end


	if sets.showBG then
		self.drop:SetBackdrop(self.bg)
		--self.drop:SetBackdropColor(self.sets.color.r, self.sets.color.g, self.sets.color.b, self.sets.color.a)
	else
		self.drop:SetBackdrop(nil)
	end
end

function component:Update(elapsed)
	if self.owner.id ~= "player" then
		return
	end

	if (self.sets.enable ~= true) or self.noUpdate then
		return
	end

	local hasVehicle = UnitHasVehicleUI(self.owner.id)
	local Form = GetShapeshiftFormID()

	local Resource  = self.lastResource
	if (self.prevClass ~= player_class) or (self.hasVehicle ~= hasVehicle) or (self.lastForm ~= Form) then
		Resource = UpdateResource(self, unit)
	end
  
	if Resource then
			Resource.update(self)
			if not self.hadResource then
				DisplayResource(self.icons)
				self.hadResource = true
			end
	else
		if self.hadResource then
			CleanResource(self.icons)
			self.hadResource  = nil
		end
	end
  
	self.lastResource = Resource
	self.lastForm = Form
	self.hasVehicle = hasVehicle
	self.prevClass = select(2, UnitClass(self.owner.id))


end

component.bg = {
	bgFile = 'Interface\\Tooltips\\UI-Tooltip-Background',
	insets = {left = {
					
							},
					right = 0, top = 0, bottom = 0},
	tile = false,
}

local anchors = {
	'TopLeft',
	'Top',
	'TopRight',
	'Right',
	'BottomRight',
	'Bottom',
	'BottomLeft',
	'Left',
	'Center',
}

local layers = {
	'BACKGROUND',
	'LOW',
	'MEDIUM',
	'HIGH',
	'DIALOG',
	'FULLSCREEN',
	'FULLSCREEN_DIALOG',
	'TOOLTIP',
}

function component:CreateMenu(menu)
	if self.owner.id ~= "player" then
		return {}
	end

	return bar.Menu.NewMenu(self, menu, 'resource', {
					{
						name = 'Enable',
						kind = 'CheckButton',
						key = 'enable',
						panel = "basic",
					},
					{
						name = 'X Offset',
						kind = 'Slider',
						key = 'x',
						min = -450,
						max = 450,
						panel = "basic",
					},
					{
						name = 'Y Offset',
						kind = 'Slider',
						key = 'y',
						min = -450,
						max = 450,
						panel = "basic",
					},
					{
						name = 'Columns',
						kind = 'Slider',
						key = 'columns',
						min = 1,
						max = max_points,
						panel = "basic",
					},
					{
						name = 'Spacing',
						kind = 'Slider',
						key = 'spacing',
						min = -13,
						max = 30,
						panel = "basic",
					},
					{
						name = 'isTopToBottom',
						kind = 'CheckButton',
						key = 'isTopToBottom',
						panel = "basic",
					},
					{
						name = 'isLeftToRight',
						kind = 'CheckButton',
						key = 'isLeftToRight',
						panel = "basic",
					},
									{
						name = 'Style',
						kind = 'Menu',
						key = 'style',
						table = {
							'ROUND',
							'SQUARE',
							'Bar',
						},
						panel = "basic",
					},
					{
						name = 'Angle',
						kind = 'Slider',
						key = 'angle',
						min = 0,
						max = 360,
						panel = "basic",
					},
					{
						name = 'Anchor',
						kind = 'Menu',
						key = 'anchor',
						table = anchors,
						panel = "advanced",
					},


					{
						name = 'Backdrop',
						kind = 'CheckButton',
						key = 'showBG',
						panel = "advanced",
					},
					{
						name = 'Backdrop Color',
						kind = 'ColorPicker',
						key = 'color',
						panel = "advanced",
					},
					{
						name = 'Scale',
						kind = 'Slider',
						key = 'scale',
						min = 25,
						max = 250,
						panel = "advanced",
					},
					{
						name = 'Height',
						kind = 'Slider',
						key = 'height',
						min = 10,
						max = 400,
						panel = "advanced",
					},	
					{
						name = 'Width',
						kind = 'Slider',
						key = 'width',
						min = 10,
						max = 400,
						panel = "advanced",
					},
					{
						name = 'Opacity',
						kind = 'Slider',
						key = 'alpha',
						min = 0,
						max = 100,
						panel = "advanced",
					},

		}, 1)
end