local bar = _G[...]
local modName = ...

local Masque = LibStub('Masque', true)


--Focus Button
local component = bar:NewComponent('focus', 'Button')

function component:New(parent)
	local name = ('%s_%s_FocusButton'):format(modName, parent.id)
	if _G[name] then
		return _G[name]
	end
	local button = self:Bind(CreateFrame('Button', name, parent.box, 'SecureActionButtonTemplate'))
	button.owner = parent
	button.icon = button:CreateTexture(nil, 'ARTWORK')
	button.icon:SetAllPoints(button)
	
	button:SetNormalTexture("")

	
	button:SetFrameLevel(parent:GetFrameLevel()+ 10)
	button:SetSize(64, 64)
	button.icon:SetTexture([[Interface\Addons\]]..modName..[[\Focus]])
	return button
end

function component:SetChecked()

end
function component:GetChecked()
	return false
end

--[[
function component:GetNormalTexture()
	return self.icon
end
--]]

component.defaults = {
	x = -42,
	y = -12,
	anchor = 'TopRight',
	enable = true,
	scale = 95,
	alpha = 100,
	usePortrait = true,
	mouseOverTooltip = true,
}

function component:Load()
	self:SetAttribute('unit', self.owner.id)
	self:RegisterForClicks('AnyUp')
	self:SetAttribute('*type1', 'target')
			
	if self.owner.id ~= 'focus' then
		self:SetAttribute('*type2', 'focus')
		self.Tooltip = 'Right-Click to Set Focus.'
	else
		self:SetAttribute('*type2', 'macro')
		self:SetAttribute('macrotext', '/clearfocus')
		self.Tooltip = 'Right-Click to Clear Focus.'
	end
	if not self.masqued then
		self.masqued = true
		
			if Masque then
				local group = Masque:Group(modName, "Focus Macros")

				group:AddButton(self, { Icon = self.icon, Border = self.IconBorder})

				if group.db.disabled then
					self:ApplyDefaultTheme(self)
				end
			end
		
	end
	self:Show()
	self:EnableMouse(true)
	self:SetScript('OnEnter', self.OnEnter)
	self:SetScript('OnLeave', self.OnExit)
end

function component:ApplyDefaultTheme()
	if sets.usePortrait then
		SetPortraitTexture(self.icon, self.owner.id)
		if (not self.icon:GetTexture()) or (self.icon:GetTexture() == "") then
			self.icon:SetTexture([[Interface\Addons\]]..modName..[[\Focus]])
		end
		self.icon:SetVertexColor(1, 1, 1)
	else
		local localizedClass, englishClass = UnitClass(self.owner.id)
		local classColor = RAID_CLASS_COLORS[englishClass]
		if classColor then
			r, g, b = classColor.r, classColor.g, classColor.b
		else
			r, g, b = 1, 1, 1
		end
		if self.icon:GetTexture() ~= [[Interface\Addons\]]..modName..[[\Focus]] then
			self.icon:SetTexture([[Interface\Addons\]]..modName..[[\Focus]])
		end
		self.icon:SetVertexColor(r, g, b)
	end
end

function component:Layout()
    if InCombatLockdown() then
        return
    end
	local sets = self.sets
	if sets.enable ~= true then
		self:Hide()
		self.noUpdate = true
		self:SetText('')
		return
	else
		self:Show()
		self.noUpdate = nil
	end
	self:ClearAllPoints()
	local x = self.sets.x/(self.sets.scale/100)
	local y = self.sets.y/(self.sets.scale/100)
	
	self:SetPoint(sets.anchor, self:GetParent(), x, y)

	if sets.scale < 25 then
		sets.scale = sets.scale *100
	end
	self:SetScale(sets.scale/100)
	self:SetAlpha(sets.alpha/100)
end

function component:Update()
	local sets = self.sets
	
	if sets.usePortrait then
		SetPortraitTexture(self.icon, self.owner.id)
		if (not self.icon:GetTexture()) or (self.icon:GetTexture() == "") then
			self.icon:SetTexture([[Interface\Addons\]]..modName..[[\Focus]])
		end
		self.icon:SetVertexColor(1, 1, 1)
	else
		local localizedClass, englishClass = UnitClass(self.owner.id)
		local classColor = RAID_CLASS_COLORS[englishClass]
		if classColor then
			r, g, b = classColor.r, classColor.g, classColor.b
		else
			r, g, b = 1, 1, 1
		end
		if self.icon:GetTexture() ~= [[Interface\Addons\]]..modName..[[\Focus]] then
			self.icon:SetTexture([[Interface\Addons\]]..modName..[[\Focus]])
		end
		self.icon:SetVertexColor(r, g, b)
	end
end

function component:OnEnter()
	UnitFrame_OnEnter(self:GetParent())
	GameTooltip:AddLine(self.Tooltip)
	GameTooltip:AppendText('')
end

function component:OnExit()
    if MouseIsOver(self:GetParent()) then
	    UnitFrame_OnEnter(self:GetParent())
    else
	    UnitFrame_OnLeave(self:GetParent())
    end
end

local layers = {
	'BACKGROUND',
	'LOW',
	'MEDIUM',
	'HIGH',
	'DIALOG',
	'FULLSCREEN',
	'FULLSCREEN_DIALOG',
	'TOOLTIP',
}

function component:CreateMenu(menu)
	return bar.Menu.NewMenu(self, menu, 'Focus', {
                    {
                        name = 'Portrait Icon',
                        kind = 'CheckButton',
                        key = 'usePortrait',
						panel = "basic",
                    },
                    {
                        name = 'Enable',
                        kind = 'CheckButton',
                        key = 'enable',
						panel = "basic",
                    },
                    {
                        name = 'Opacity',
                        kind = 'Slider',
                        key = 'alpha',
                        min = 0,
                        max = 100,
						panel = "basic",
                    },
                    {
                        name = 'Tooltip',
                        kind = 'CheckButton',
                        key = 'mouseOverTooltip',
						panel = "advanced",
                    },
                    {
                        name = 'X Offset',
                        kind = 'Slider',
                        key = 'x',
                        min = -400,
                        max = 400,
						panel = "advanced",
                    },
                    {
                        name = 'Y Offset',
                        kind = 'Slider',
                        key = 'y',
                        min = -400,
                        max = 400,
						panel = "advanced",
                    },
                    {
                        name = 'Anchor',
                        kind = 'Menu',
                        key = 'anchor',
                        table = {
                            'TopLeft',
                            'Top',
                            'TopRight',
                            'Right',
                            'BottomRight',
                            'Bottom',
                            'BottomLeft',
                            'Left',
                            'Center',
                        },
						panel = "advanced",
                    },
                    {
                        name = 'Scale ',
                        kind = 'Slider',
                        key = 'scale',
                        min = 25,
                        max = 250,
						panel = "advanced",
                    },

        }, 1)
end



